﻿<%@ WebHandler Language="C#" Class="AutoComplete" %>

using System;
using System.Linq;
using System.Web;

public class AutoComplete : IHttpHandler
{

    public void ProcessRequest(HttpContext context)
    {
        context.Response.ContentType = "text/plain";
        context.Response.Write(GetQuerySuggestions(context));
    }

    protected string GetQuerySuggestions(HttpContext context)
    {
        var provider = "solr";
        if (!String.IsNullOrEmpty(context.Request.QueryString["provider"]))
        {
            provider = context.Request.QueryString["provider"];
        }
        System.Web.Script.Serialization.JavaScriptSerializer jss = new System.Web.Script.Serialization.JavaScriptSerializer();

        switch (provider)
        {
            case "solr":
                return EKQueryHelper(context);
            case "searchserver":
                return EKQueryHelper(context, false);
            case "google":
                return GoogleBingQueryHelper(context);
            case "bing":
                return GoogleBingQueryHelper(context, false);
            default:
                //prefetch
                return Prefetch();
        }
    }

    protected string Prefetch()
    {
        System.Web.Script.Serialization.JavaScriptSerializer jss = new System.Web.Script.Serialization.JavaScriptSerializer();
        System.Collections.Generic.List<string> prefetchvalue = new System.Collections.Generic.List<string>();
        //add as many of these as needed as prefetch values meaning you can define suggested search terms.
        prefetchvalue.Add("xslt");
        prefetchvalue.Add("code");
        return jss.Serialize(prefetchvalue);
    }

    protected string GoogleBingQueryHelper(HttpContext context, bool google = true)
    {
        System.Web.Script.Serialization.JavaScriptSerializer jss = new System.Web.Script.Serialization.JavaScriptSerializer();
        string query = context.Server.HtmlDecode(context.Request.QueryString["query"]);
        if (!query.ToLower().Contains("@"))
        {
            string requestURL = "http://suggestqueries.google.com/complete/search?q=" + query + "&output=firefox";
            if (!google)
            {
                requestURL = "http://api.bing.com/osjson.aspx?query=" + query;
            }
            System.Net.WebRequest request = System.Net.WebRequest.Create(requestURL);
            // Get the response.
            System.Net.WebResponse response = request.GetResponse();
            // Get the stream containing content returned by the server.
            System.IO.Stream dataStream = response.GetResponseStream();
            // Open the stream using a StreamReader for easy access.
            System.IO.StreamReader reader = new System.IO.StreamReader(dataStream);
            // Read the content.
            string responseFromServer = reader.ReadToEnd();
            System.Collections.Generic.List<string> queryResult = new System.Collections.Generic.List<string>();
            //Ektron.Newtonsoft.Json.Linq.JArray jObject = Ektron.Newtonsoft.Json.Linq.JArray.Parse(responseFromServer);
            return jss.Serialize(responseFromServer);
        }
        return jss.Serialize("");
    }

    protected string EKQueryHelper(HttpContext context, bool solr = true)
    {
        int itemsToRetrieve = 0;
        int.TryParse(context.Request.QueryString["maxResults"], out itemsToRetrieve);
        System.Web.Script.Serialization.JavaScriptSerializer jss = new System.Web.Script.Serialization.JavaScriptSerializer();
        string query = context.Server.HtmlDecode(context.Request.QueryString["query"]);
        if (!query.ToLower().Contains("@"))
        {
            if (!query.EndsWith(" "))
            {
                Ektron.Cms.Framework.Search.QueryPropositionManager qpmanager = new Ektron.Cms.Framework.Search.QueryPropositionManager();
                string appendQuery = "";
                if (query.Contains(" "))
                {
                    appendQuery = query.Substring(query.LastIndexOf(" "));
                    query = query.Remove(query.LastIndexOf(appendQuery));

                    if (solr)
                    {
                        Ektron.Cms.Search.QueryCompletionResponse qs = qpmanager.GetQueryCompletions(appendQuery.Trim(), itemsToRetrieve);
                        qs.QueryCompletions.ToList().ForEach(a => a.Completion = query + " " + a.Completion);
                        return jss.Serialize(qs.QueryCompletions);
                    }
                    else
                    {
                        Ektron.Cms.Search.QuerySuggestionResponse qs = qpmanager.GetQuerySuggestions(query.Trim(), itemsToRetrieve);
                        qs.QuerySuggestions.ToList().ForEach(a => a.Suggestion = query + " " + a.Suggestion);
                        return jss.Serialize(qs.QuerySuggestions);
                    }
                }
                else
                {
                    if (solr)
                    {
                        var qs = qpmanager.GetQueryCompletions(query.Trim(), itemsToRetrieve);
                        return jss.Serialize(qs.QueryCompletions);
                    }
                    else
                    {
                        var qs = qpmanager.GetQuerySuggestions(query.Trim(), itemsToRetrieve);
                        return jss.Serialize(qs.QuerySuggestions);
                    }
                }
            }
        }
        return jss.Serialize("");
    }

    public bool IsReusable
    {
        get
        {
            return false;
        }
    }

}